Module: SafeAWS
- Defined in:
- lib/SafeAWS/EC2.rb,
lib/SafeAWS/S3.rb,
lib/SafeAWS/SQS.rb,
lib/SafeAWS/SimpleDB.rb
Overview
SafeAWS wraps the AWS module in exception catcher blocks, so that any exceptions that are thrown do not affect the caller.
The SafeEC2, SafeSQS, SafeSimpleDB, and SafeS3 log any errors that they encounter, so that they can be examined later.